#ifndef FTPP_EXCEPTION_H
#define FTPP_EXCEPTION_H

#include <exception>
#include <string>
#include <sstream>
#include <freetype2/ft2build.h>
#include FT_FREETYPE_H

namespace freetype {

   const char* get_error_string(FT_Error error);

   class Exception: public std::exception {
      public:
	 Exception(FT_Error error) : error(error) {
	 }
	 Exception(FT_Error error, const std::string& method) :
	       error(error), method(method) {
	 }
	 Exception(FT_Error error, const std::string& method,
	       const std::string& pathname) :
	       error(error), method(method), pathname(pathname) {
	 }
	 virtual ~Exception() noexcept {
	 }
	 virtual const char* what() const noexcept {
	    std::ostringstream osmsg;
	    if (method.size() > 0) {
	       osmsg << method << ": ";
	    }
	    if (pathname.size() > 0) {
	       osmsg << pathname << ": ";
	    }
	    const char* error_string = get_error_string(error);
	    if (error_string) {
	       osmsg << error_string;
	    } else {
	       osmsg << "error code " << error;
	    }
	    msg = osmsg.str();
	    return msg.c_str();
	 }
      private:
	 FT_Error error;
	 std::string method;
	 std::string pathname;
	 mutable std::string msg;
   };

} // namespace freetype

#endif
