#ifndef FTPP_GLYPH_H
#define FTPP_GLYPH_H

#include <cassert>
#include <utility>
#include <freetype2/ft2build.h>
#include FT_FREETYPE_H
#include FT_GLYPH_H
#include <ftpp/exception.hpp>
#include <ftpp/bitmap.hpp>

namespace freetype {

class Glyph {
   public:
      Glyph() noexcept : library(nullptr), glyph(nullptr) {
      }
      Glyph(FT_Library library, FT_Glyph source) : library(library) {
	 FT_Error error = FT_Glyph_Copy(source, &glyph);
	 if (error) throw Exception(error, "FT_Glyph_Copy");
      }
      Glyph(FT_Library library, FT_GlyphSlot glyph_slot) :
	    library(library) {
	 FT_Error error = FT_Get_Glyph(glyph_slot, &glyph);
	 if (error) throw Exception(error, "FT_Get_Glyph");
      }
      Glyph(Glyph&& other) noexcept {
	 swap(other);
      }
      ~Glyph() {
	 if (glyph) FT_Done_Glyph(glyph);
      }
      Glyph& operator=(const Glyph& other) {
	 if (glyph) {
	    FT_Done_Glyph(glyph);
	    glyph = nullptr;
	 }
	 FT_Error error = FT_Glyph_Copy(other.glyph, &glyph);
	 if (error) throw Exception(error, "FT_Glyph_Copy");
	 library = other.library;
      }
      Glyph& operator=(Glyph&& other) noexcept {
	 swap(other);
	 return *this;
      }
      void swap(Glyph& other) noexcept {
	 std::swap(library, other.library);
	 std::swap(glyph, other.glyph);
      }
   private:
      FT_Library library;
      FT_Glyph glyph;
};

} // namespace freetype

#endif
