#ifndef FTPP_LIBRARY_H
#define FTPP_LIBRARY_H

#include <utility>
#include <freetype2/ft2build.h>
#include FT_FREETYPE_H
#include <ftpp/exception.hpp>
#include <ftpp/face.hpp>

namespace freetype {

class Library {
   public:
      Library() {
	 FT_Error error = FT_Init_FreeType(&library);
	 if (error) throw Exception(error, "FT_Init_FreeType");
      }
      Library(Library&& other) noexcept {
	 swap(other);
      }
      Library& operator=(const Library& other) = delete;
      Library& operator=(Library&& other) noexcept {
	 swap(other);
	 return *this;
      }
      void swap(Library& other) noexcept {
	 std::swap(library, other.library);
      }
      Face new_face(const char* filename,
	    FT_Long face_index = 0) {
	 return Face(library, filename, face_index);
      }
      ~Library() {
	 if (library) FT_Done_FreeType(library);
      }

   private:
      FT_Library library;
};

} // namespace freetype

#endif
