#include <cstdlib>
#include <cstring>
#include <exception>
#include <iostream>
#include <sstream>
#include <printf.hpp>
#include <ftpp/library.hpp>
#include "utf8.hpp"

using namespace freetype;

int main(int argc, char** argv) {
   const char* cmdname = *argv++; --argc;
   if (argc != 2) {
      std::cerr << "Usage: " << cmdname <<
	 " font text" << std::endl;
      exit(1);
   }
   const char* filename = *argv++; --argc;

   const char* text = *argv++; --argc;
   std::size_t textlen = strlen(text);

   Library lib;
   try {
      Face face = lib.new_face(filename);
      if (face.has_kerning()) {
	 char32_t previous = 0;
	 for (auto charcode: codepoint_range(text, text + textlen)) {
	    if (previous) {
	       auto index1 = face.get_glyph_index(previous);
	       auto index2 = face.get_glyph_index(charcode);
	       if (!index1 || !index2) continue;
	       auto akerning = face.get_kerning(index1, index2,
		  FT_KERNING_UNSCALED);
	       fmt::printf("0x%04x 0x%04x %d\n", previous, charcode,
		  akerning.x.to_raw());
	    }
	    previous = charcode;
	 }
      } else {
	 fmt::printf(std::cerr, "%s: %s has no kerning\n",
	    cmdname, filename);
      }
   } catch(Exception except) {
      std::cerr << cmdname << ": " << except.what() << std::endl;
   }
}
