#include <cstdlib>
#include <exception>
#include <iostream>
#include <sstream>
#include <ftpp/library.hpp>

using namespace freetype;

int main(int argc, char** argv) {
   const char* cmdname = *argv++; --argc;
   if (argc != 4) {
      std::cerr << "Usage: " << cmdname <<
	 " font code size resolution" << std::endl;
      exit(1);
   }
   const char* filename = *argv++; --argc;

   const char* code = *argv++; --argc;
   FT_ULong charcode;
   {
      std::istringstream abuf(code);
      if (!(abuf >> charcode)) {
	 std::cerr << cmdname << ": invalid code: " << code << std::endl;
	 exit(1);
      }
   }

   const char* resolution_string = *argv++; --argc;
   FT_ULong resolution; /* in dpi */
   {
      std::istringstream abuf(resolution_string);
      if (!(abuf >> resolution)) {
	 std::cerr << cmdname << ": invalid resolution: "
	    << resolution_string << std::endl;
	 exit(1);
      }
   }

   const char* size_string = *argv++; --argc;
   f26dot6 size;
   {
      std::istringstream abuf(size_string);
      if (!(abuf >> size)) {
	 std::cerr << cmdname << ": invalid size: " << size_string << std::endl;
	 exit(1);
      }
   }

   Library lib;
   try {
      Face face = lib.new_face(filename);
      face.set_char_size(0, size, resolution, resolution);
      auto index = face.get_glyph_index(charcode);
      if (!index) {
	 std::cerr << cmdname << ": code not supported: "
	    << charcode << std::endl;
	 exit(1);
      }
      std::cout << "units per em = " << face.get_units_per_EM() << std::endl;
      std::cout << "bbox = " << face.get_bbox() << std::endl;

      face.load_glyph(index);
      face.render_glyph(FT_RENDER_MODE_MONO);
      Bitmap bitmap = face.get_bitmap();
      std::cout << "rows = " << bitmap.get_num_rows() << std::endl;
      std::cout << "width = " << bitmap.get_width() << std::endl;
      std::cout << "advance = " << face.get_advance() << std::endl;
      std::cout << "left = " << face.get_left() << std::endl;
      std::cout << "top = " << face.get_top() << std::endl;
      for (unsigned int i = 0; i < bitmap.get_num_rows(); ++i) {
	 for (unsigned int j = 0; j < bitmap.get_width(); ++j) {
	    if (bitmap(i, j)) {
	       std::cout << "#";
	    } else {
	       std::cout << " ";
	    }
	 }
	 std::cout << std::endl;
      }
   } catch(Exception except) {
      std::cerr << cmdname << ": " << except.what() << std::endl;
   }
}
